@@ -40,26 +40,29 @@ typedef struct {
 } texcape;
 
 typedef struct {
-	char pos;	/* position in cell (align) */
-	char block;	/* CSV delimitor if any */
-	char sep;	/* CSV separator */
-	int cols;	/* CSV columns (found at runtime) */
-	int chars;	/* CSV max data length found at runtime */
-	int lines;	/* rows per LaTeX tabular */
-	int guess;	/* guess or not the CSV format */
-	int header;	/* put LaTeX document header or not */
-	int escape;	/* escape TeX control chars or not */
+	char block;		/* CSV delimitor if any */
+	char sep;		/* CSV separator */
+	unsigned int cols;	/* CSV columns */
+	unsigned int chars;	/* CSV max data length */
+	unsigned int rows;	/* CSV total number of lines */
+	char pos;		/* position in cell (align) */
+	unsigned int lines;	/* rows per LaTeX tabular */
+	unsigned int guess;	/* guess or not the CSV format */
+	unsigned int header;	/* put LaTeX document header or not */
+	unsigned int red;	/* table reduction level (from 1 to 4)*/
+	unsigned int longtable;	/* use package longtable */
+	unsigned int escape;	/* escape TeX control chars or not */
 	char* clrrow;	/* row graylevel (from 0 to 1) */
 	texcape* tex;	/* TeX escapes */
-	int red;	/* table reduction level (from 1 to 4)*/
 } config;
 
-#define MAXINT (int)((unsigned int)(-1) / 2)
+#define MAXUINT ((unsigned int)(-1))
 
 void rtfm(char* prog) {
    printf("%s translates a csv file to a LaTeX file\n", basename(prog));
    printf("Example: %s january_stats.csv > january_stats.tex\n", basename(prog));
    printf("Usage: %s [--nohead] (LaTeX) no document header: useful for inclusion\n", basename(prog));
+   printf("	[--longtable] (LaTeX) use package longtable: useful for long input\n");
    printf("	[--noescape] (LaTeX) do not escape text: useful for mixed CSV/TeX input\n");
    printf("	[--guess] (CSV) guess separator and block |\n"
 		"	[--separator <(c)omma|(s)emicolon|(t)ab|s(p)ace|co(l)on>] (CSV's comma)\n"
@@ -69,6 +72,7 @@ void rtfm(char* prog) {
    printf("	[--colorrows graylevel] (LaTeX) alternate gray rows (e.g: 0.75)\n");
    printf("	[--reduce level] (LaTeX) reduce table size (e.g: 1)\n");
    printf("		csv_file.csv\n");
+   printf("The \"longtable\" option needs the {longtable} LaTeX package\n");
    printf("The \"colorrows\" option needs the {colortbl} LaTeX package\n");
    printf("The \"reduce\" option needs the {relsize} LaTeX package\n");
    return;
@@ -94,13 +98,14 @@ config* parseOptions (config* conf, int argc, char **argv) {
 		{"separator", 1, NULL, 's'},
 		{"colorrows", 1, NULL, 'c'},
 		{"reduce",    1, NULL, 'r'},
+		{"longtable", 0, NULL, 't'},
 		{NULL, 0, NULL, 0} /* marks end-of-list */
 	};
 #endif
 #if defined USE_GETOPT
-	while ((opt = getopt (argc, argv, "hvgnx?b:l:p:s:c:r:")) != EOF) {
+	while ((opt = getopt (argc, argv, "hvgnxt?b:l:p:s:c:r:")) != EOF) {
 #else
-	while ((opt = getopt_long (argc, argv, "hvgnx?b:l:p:s:c:r:", long_options, &longopt_index)) > 0) {
+	while ((opt = getopt_long (argc, argv, "hvgnxt?b:l:p:s:c:r:", long_options, &longopt_index)) > 0) {
 #endif
 		switch (opt) {
 			case '?':
@@ -111,6 +116,9 @@ config* parseOptions (config* conf, int argc, char **argv) {
 			case 'g': /* guess the CSV */
 					conf->guess=1;
 					break;
+			case 't': /* use package longtable */ /* thanks to <Christof.Bodner@infineon.com> */
+					conf->longtable=1;
+					break;
 			case 'b': /* csv block delimiter */
 					if(optarg[0] == 'q')
 						conf->block = '\'';
@@ -227,15 +235,16 @@ int guessCSV(config* conf, FILE* in) {
 	return(0);
 }
 
-void getMaximums(config* conf, FILE* in, int* maxchar, int* maxcol) {
+void getMaximums(config* conf, FILE* in)  {
 /* gets the number of cols and chars of a csv file assuming a separator */
 	int token=0;
-	int curcol=0;
-	int curchar=0;
-	int inblock=0;
-	
-	(*maxchar)=0;
-	(*maxcol)=0;
+	unsigned int curcol=0;
+	unsigned int curchar=0;
+	unsigned int inblock=0;
+	/* init */
+	conf->chars=0;
+	conf->cols=0;
+	conf->rows=0;
 
 	while (token != EOF) {
 		token=getc(in);
@@ -248,8 +257,9 @@ void getMaximums(config* conf, FILE* in, int* maxchar, int* maxcol) {
 		/* decide the maximums */
 		if (token == '\n') {
 			curcol++;
-			(*maxcol)=((*maxcol)<curcol)?curcol:(*maxcol);
-			(*maxchar)=((*maxchar)<curchar)?curchar:(*maxchar);
+			conf->cols=(conf->cols<curcol)?curcol:conf->cols;
+			conf->chars=(conf->chars<curchar)?curchar:conf->chars;
+			conf->rows++;
 			curcol=0;
 			curchar=0;
 			inblock=0; /* reset block state */
@@ -284,10 +294,12 @@ void doTeXsub(config* conf, char newsep, FILE* in, FILE* out) {
 	int numcols;
 	int lines;
 	int inblock=0;
+	int csvrows;
 
 	max=numcols=conf->cols;
+	csvrows=conf->rows;
 	/* choose infinity when conf->lines is 0 */
-	lines=(conf->lines)?conf->lines:MAXINT;
+	lines=(conf->lines)?conf->lines:MAXUINT;
 
 	while (token!=EOF) {
 		token=getc(in);
@@ -309,22 +321,28 @@ void doTeXsub(config* conf, char newsep, FILE* in, FILE* out) {
 			fprintf(out,"\\hline\n"); /* TeX draw hline */
 			numcols=max; /* reset numcols */
 			lines--;
+			csvrows--;
 			/* put a colored row or not (alternate) */
 			if (conf->clrrow && (lines % 2)){
 				fprintf(out,"\\colorrow ");
 			}
-			/* if the LaTeX tabular is full create a new one */
-			/* FIXME: empty tabular when sum match CSV lines */
-			if (!lines){
-				fprintf(out,"\\end{tabular}\n");
-				fprintf(out,"\\begin{tabular}{|");
+			/* if the LaTeX tabular is full create a new one, except if no more row */
+			if (!lines && csvrows){
+				if (conf->longtable) {
+				  fprintf(out,"\\end{longtable}\n");
+				  fprintf(out, "\\newline\n");
+				  fprintf(out,"\\begin{longtable}{|\n"); }
+				else {
+				  fprintf(out,"\\end{tabular}\n");
+				  fprintf(out, "\\newline\n");
+				  fprintf(out,"\\begin{tabular}{|"); }
 				while(numcols--)
 					fprintf(out, "%c|", conf->pos);
 				fprintf(out, "}\n");
 				fprintf(out, "\\hline\n");
 				numcols=max;
-				lines=(conf->lines)?conf->lines:MAXINT;
-			}
+				lines=(conf->lines)?conf->lines:MAXUINT;
+			} /* else end of CSV data */
 			continue;
 		}
 
@@ -375,8 +393,8 @@ void doTeXdoc(config* conf, FILE* in, FILE* out) {
 /* prepares the LaTeX tabular layout */
 	int maxcols;
 	int numcols;
-	char* relsize[5] = {"0", "0.5", "1", "2", "4"};
-	char* tabcolsep[5] = {"0", "0.05", "0.1", "0.2", "0.4"};
+	char* relsize[5] = {"0", "0.5", "1", "2", "4"}; /* LaTeX relsize good values */
+	char* tabcolsep[5] = {"0", "0.05", "0.1", "0.2", "0.4"}; /* LaTeX tabcolsep good values */
 
 	numcols=maxcols=conf->cols;
 	if(conf->header){
@@ -389,6 +407,9 @@ void doTeXdoc(config* conf, FILE* in, FILE* out) {
 		if (conf->clrrow){
 			fprintf(out,"\\usepackage{colortbl}\n");
 		}
+		if (conf->longtable){
+			fprintf(out,"\\usepackage{longtable}\n");
+		}
 		fprintf(out, "\\begin{document}\n");
 	}
 	if (conf->clrrow){
@@ -399,13 +420,20 @@ void doTeXdoc(config* conf, FILE* in, FILE* out) {
 		fprintf(out,"\\relsize{-%s}\n", relsize[conf->red]);
 		fprintf(out,"\\addtolength\\tabcolsep{-%sem}\n", tabcolsep[conf->red]);
 	}
-	fprintf(out, "\\begin{tabular}{|");
+	if (conf->longtable)
+	  fprintf(out, "\\begin{longtable}{|");
+  	else
+	  fprintf(out, "\\begin{tabular}{|");
 	while(numcols--)
 		fprintf(out, "%c|",conf->pos); /* position in cell */
 	fprintf(out, "}\n");
 	fprintf(out, "\\hline\n");
-	doTeXsub(conf, '&', in, out); /* & is LaTeX sep */
-	fprintf(out, "\\end{tabular}\n");
+	doTeXsub(conf, '&', in, out); /* & is LaTeX separator */
+	if (conf->longtable) {
+	  fprintf(out, "\\end{longtable}\n");
+  	} else {
+	  fprintf(out, "\\end{tabular}\n");
+	}
 	if (conf->red){
 		fprintf(out,"\\addtolength\\tabcolsep{+%sem}\n", tabcolsep[conf->red]);
 		fprintf(out,"\\relsize{+%s}\n", relsize[conf->red]);
@@ -428,8 +456,9 @@ int main (int argc, char **argv) {
 	}
 	conf=(config*)malloc(sizeof(config));
 	/* defaults (ensure init): */
-	conf->cols=1;        /* if getMaximums fails */
-	conf->chars=0;       /* must be 0 */
+	conf->cols=1;        /* CSV: if getMaximums fails */
+	conf->rows=0;         /* CSV: must be 0 */
+	conf->chars=0;       /* CSV: must be 0 */
 	conf->pos='l';       /* usual; LaTeX */
 	conf->lines=40;      /* usual; LaTeX */
 	conf->guess=0;       /* usual */
@@ -439,6 +468,7 @@ int main (int argc, char **argv) {
 	conf->escape=1;      /* usual; LaTeX */
 	conf->clrrow=NULL;   /* default; LaTeX */
 	conf->red=0;         /* default; LaTeX */
+	conf->longtable=0;   /* default; without package longtable */
 	
 	/* TeX charaters to escape */
 	conf->tex=(texcape*)malloc(sizeof(texcape));
@@ -458,7 +488,7 @@ int main (int argc, char **argv) {
 		}
 		rewind(fp);
 	}
-	getMaximums(conf, fp, &conf->chars, &conf->cols);
+	getMaximums(conf, fp);
 	rewind(fp);
 	doTeXdoc(conf, fp, stdout);
 	free(conf->tex);
